/*	$NetBSD: zfs_component.c,v 1.3 2019/05/07 08:51:09 hannken Exp $	*/

/*
 * Copyright (c) 2009 Antti Kantee.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD: zfs_component.c,v 1.3 2019/05/07 08:51:09 hannken Exp $");

#include <sys/param.h>
#include <sys/conf.h>
#include <sys/device.h>
#include <sys/stat.h>

#include <rump-sys/kern.h>
#include <rump-sys/vfs.h>

RUMP_COMPONENT(RUMP_COMPONENT_VFS)
{
	extern const struct bdevsw zfs_bdevsw;
	extern const struct cdevsw zfs_cdevsw;
	devmajor_t bmaj, cmaj;
	int error, exist;

	/* go, mydevfs */
	bmaj = cmaj = -1;

	if ((error = devsw_attach("zfs", &zfs_bdevsw, &bmaj,
	    &zfs_cdevsw, &cmaj)) != 0 && error != EEXIST)
		panic("cannot attach zfs: %d", error);
	exist = (error == EEXIST);

	if ((error = rump_vfs_makeonedevnode(S_IFCHR,
	    "/dev/zfs", cmaj, 0)) != 0)
		panic("cannot create zfs dev nodes: %d", error);

	if (!exist)
		devsw_detach(&zfs_bdevsw, &zfs_cdevsw);
}
